#!/usr/bin/env Rscript
args <- commandArgs(trailingOnly=TRUE)
if (length(args)==0){
  args <- 1
}

# Needed packages:
library("methods")
library("lvnet")
library("mvtnorm")
library("dplyr")
library("ggplot2")
library("pracma")

# parSim function:
source("parSim.R")

# Extra functions
source("funs.R")

### RUN THE SIMULATION ###
parSim(
  # Conditions:
  # Single values can be changed into a vector to test multiple conditions
  sampleSize = c(100,250,500,1000,2500), # sample size
  nLatent = 8, # Number of latents
  nIndicator = 3, # Number of indicators per latent
  
  # Setup:
  name = paste0("LNM_lasso_run2_",args[1]),
  write = TRUE, # If FALSE, simulation results are stored to a data frame. Now, results are written to a text file.
  nCores = 16, # Number of cores to use. Set to 1 for debugging options
  reps = 10, # Number of times all conditions should be repeated. When using a high-performance cluster, my typical setup is to leave this to 1 and submit this file 100 times.
  export = c("simGraph_WS","forcePositive"), # Functions that should be exported to the cluster

  # Codes:
  expression = {
    suppressMessages(library("lvnet"))
    suppressMessages(library("mvtnorm"))
    suppressMessages(library("dplyr"))
    suppressMessages( library("ggplot2"))
    suppressMessages( library("pracma"))
    
    
    nManifest <- nLatent*nIndicator
    totEdges <- (nLatent*(nLatent-1)) / 2
    nEdge <- totEdges
    
    # Construct lambda:
    Lambda <- matrix(0,nManifest, nLatent)
    for (j in 1:nLatent){
      Lambda[nIndicator*(j-1)+(1:nIndicator),j] <- 1
    }
    
    # Construct Theta:
    Theta <- diag(nManifest)
    
    # Construct latent network:
    O_Psi <- simGraph_WS(nLatent, 0)
    
    # Sigma:
    Psi <- cov2cor(solve(eye(nLatent) - O_Psi))
    Sigma <- Lambda %*% Psi %*% t(Lambda) + Theta
    
    # Simulate data:
    Data <- rmvnorm(sampleSize, sigma=Sigma)
    
    # Fit model:
    t0 <- Sys.time()
    Res <- lvnetLasso(cov(Data), lassoMatrix = "omega_psi", sampleSize = nrow(Data),lambda = ifelse(Lambda==1,NA,0),
                      verbose = FALSE, refit = FALSE, criterion = "aic")
    timeDiff <- as.numeric(difftime(Sys.time(), t0,units = "mins"))
    

    Results <- lapply(c("AIC","BIC","EBIC"), function(x){
      
      if (x == "AIC"){
        Res <- lassoSelect(Res, aic, refit=FALSE)
      }
      
      if (x == "BIC"){
        Res <- lassoSelect(Res, bic, refit=FALSE)
      }
      
      if (x == "EBIC"){
        Res <- lassoSelect(Res, ebic, refit=FALSE)
      }
      
      ### STORE RESULTS ###
      SimulationResults <- data.frame(criterion = x)
      
      # time difference:
      SimulationResults$timeDiff <- timeDiff
      
      
      # Chisq:
      SimulationResults$chisq <- Res$best$fitMeasures$chisq
      
      # df:
      SimulationResults$df <- Res$best$fitMeasures$df
      
      # p-value:
      SimulationResults$pvalue <- Res$best$fitMeasures$pvalue
      
      # RMSEA:
      SimulationResults$rmsea <- Res$best$fitMeasures$rmsea
      
      # CFI:
      SimulationResults$cfi <- Res$best$fitMeasures$cfi
      
      # SRMR:
      SimulationResults$srmr <-  Res$best$fitMeasures$srmr
      
      # Omega_theta:
      res_omega_psi <- Res$best$matrices$omega_psi
      res_omega_psi[abs(res_omega_psi) < 1e-4] <- 0
      
      # Equal?
      SimulationResults$correctModel <- all((res_omega_psi == 0) == ( O_Psi == 0))
      
      # True positives:
      TruePos <- sum(res_omega_psi[upper.tri(res_omega_psi)] != 0 &  O_Psi[upper.tri( O_Psi)] != 0)
      
      # False pos:
      FalsePos <- sum(res_omega_psi[upper.tri(res_omega_psi)] != 0 &  O_Psi[upper.tri( O_Psi)] == 0)
      
      # True Neg:
      TrueNeg <- sum(res_omega_psi[upper.tri(res_omega_psi)] == 0 &  O_Psi[upper.tri( O_Psi)] == 0)
      
      # False Neg:
      FalseNeg <- sum(res_omega_psi[upper.tri(res_omega_psi)] == 0 &  O_Psi[upper.tri( O_Psi)] != 0)
      
      ### Sensitivity:
      SimulationResults$sensitivity <- TruePos / (TruePos + FalseNeg)
      
      # Specificity:
      SimulationResults$specificity <- TrueNeg / (TrueNeg + FalsePos)

      SimulationResults
      
    })
    
    Results <- do.call(rbind,Results)
    return(Results)
  }
)