#!/usr/bin/env Rscript
args <- commandArgs(trailingOnly=TRUE)
if (length(args)==0){
  args <- 1
}

# Needed packages:
library("methods")
library("lvnet")
library("mvtnorm")
library("dplyr")
library("ggplot2")
library("pracma")

# parSim function:
source("parSim.R")

# Extra functions
source("funs.R")

### RUN THE SIMULATION ###
parSim(
  # Conditions:
  # Single values can be changed into a vector to test multiple conditions
  sampleSize = c(50,100,250,500,1000), # sample size
  nLatent = 4, # Number of latents
  nIndicator = 3, # Number of indicators per latent
  criterion = c("AIC","BIC","chisq","EBIC"), # Add or change to "chisq" and "AIC" for different conditions
  
  # Setup:
  name = paste0("LNM_stepwise_run4_",args[1]),
  write = TRUE, # If FALSE, simulation results are stored to a data frame. Now, results are written to a text file.
  nCores = 16, # Number of cores to use. Set to 1 for debugging options
  reps = 10, # Number of times all conditions should be repeated. When using a high-performance cluster, my typical setup is to leave this to 1 and submit this file 100 times.
  export = c("simGraph_WS","forcePositive"), # Functions that should be exported to the cluster

  # Codes:
  expression = {
    suppressMessages(library("lvnet"))
    suppressMessages(library("mvtnorm"))
    suppressMessages(library("dplyr"))
    suppressMessages( library("ggplot2"))
    suppressMessages( library("pracma"))
    
    
    nManifest <- nLatent*nIndicator
    totEdges <- (nLatent*(nLatent-1)) / 2
    nEdge <- totEdges
    
    # Construct lambda:
    Lambda <- matrix(0,nManifest, nLatent)
    for (j in 1:nLatent){
      Lambda[nIndicator*(j-1)+(1:nIndicator),j] <- 1
    }
    
    # Construct Theta:
    Theta <- diag(nManifest)
    
    # Construct latent network:
    O_Psi <- simGraph_WS(nLatent,p = 0)

    # Sigma:
    Psi <- cov2cor(solve(eye(nLatent) - O_Psi))
    Sigma <- Lambda %*% Psi %*% t(Lambda) + Theta
    
    # Simulate data:
    Data <- rmvnorm(sampleSize, sigma=Sigma)
    
    # Fit model:
    t0 <- Sys.time()
    Res <- lvnetSearch(forcePositive(cov(Data)), matrix = "omega_psi", sampleSize = nrow(Data),lambda = ifelse(Lambda==1,NA,0),
                       start = "full", criterion = tolower(criterion), verbose = FALSE, maxChange = 1)
    timeDiff <- as.numeric(difftime(Sys.time(), t0,units = "mins"))
    
    ResultsList <- list()
    
    ResultsList$timeDiff <- timeDiff
    
    
    # Chisq:
    ResultsList$chisq <- Res$best$fitMeasures$chisq
    
    # df:
    ResultsList$df <- Res$best$fitMeasures$df
    
    # p-value:
    ResultsList$pvalue <- Res$best$fitMeasures$pvalue
    
    # RMSEA:
    ResultsList$rmsea <- Res$best$fitMeasures$rmsea
    
    # CFI:
    ResultsList$cfi <- Res$best$fitMeasures$cfi
    
    # SRMR:
    ResultsList$srmr <-  Res$best$fitMeasures$srmr
    
    # Omega_psi:
    res_omega_psi <- Res$best$matrices$omega_psi
    
    # Equal?
    ResultsList$correctModel <- all((res_omega_psi == 0) == (O_Psi == 0))
    
    # True positives:
    TruePos <- sum(res_omega_psi[upper.tri(res_omega_psi)] != 0 & O_Psi[upper.tri(O_Psi)] != 0)
    
    # False pos:
    FalsePos <- sum(res_omega_psi[upper.tri(res_omega_psi)] != 0 & O_Psi[upper.tri(O_Psi)] == 0)
    
    # True Neg:
    TrueNeg <- sum(res_omega_psi[upper.tri(res_omega_psi)] == 0 & O_Psi[upper.tri(O_Psi)] == 0)
    
    # False Neg:
    FalseNeg <- sum(res_omega_psi[upper.tri(res_omega_psi)] == 0 & O_Psi[upper.tri(O_Psi)] != 0)
    
    ### Sensitivity:
    ResultsList$sensitivity <- TruePos / (TruePos + FalseNeg)
    
    # Specificity:
    ResultsList$specificity <- TrueNeg / (TrueNeg + FalsePos)

    ResultsList
  }
)