#!/usr/bin/env Rscript
args <- commandArgs(trailingOnly=TRUE)
if (length(args)==0){
  args <- 1
}

# Needed packages:
library("methods")
library("lvnet")
library("mvtnorm")
library("dplyr")
library("ggplot2")
library("pracma")

# parSim function:
source("parSim.R")

# Extra functions
source("funs.R")

### RUN THE SIMULATION ###
parSim(
  # Conditions:
  # Single values can be changed into a vector to test multiple conditions
  sampleSize = c(100,250,500,1000,2500), # sample size
  nLatent = 4, # Number of latents

  # Setup:
  name = paste0("RNM_lasso_run2_",args[1]),
  write = TRUE, # If FALSE, simulation results are stored to a data frame. Now, results are written to a text file.
  nCores = 16, # Number of cores to use. Set to 1 for debugging options
  reps = 10, # Number of times all conditions should be repeated. When using a high-performance cluster, my typical setup is to leave this to 1 and submit this file 100 times.
  export =  c("simGraph_WS","forcePositive"), # Functions that should be exported to the cluster

  # Codes:
  expression = {
    suppressMessages(library("lvnet"))
    suppressMessages( library("mvtnorm"))
    suppressMessages(library("dplyr"))
    suppressMessages(library("ggplot2"))
    suppressMessages(library("pracma"))
    suppressMessages(library("igraph"))
    
    # Set number of indicators per latent:
    nManifest <- 20
    if (nLatent==1){
      nIndicator <- 20
    } else if (nLatent == 2){
      nIndicator <- c(10, 10)
    } else if (nLatent == 3){
      nIndicator <- c(7,7,6)
    } else if (nLatent == 4){
      nIndicator <- c(5,5,5,5)
    } else stop("Incorrect number of indicators")
    
    # nManifest <- nLatent*nIndicator
    totEdges <- (nManifest*(nManifest-1)) / 2

    # Construct lambda:
    Lambda <- matrix(0,nManifest, nLatent)
    c <- 0
    for (j in 1:nLatent){
      Lambda[c + 1:nIndicator[j],j] <- 1
      c <- c + nIndicator[j]
    }
    
    # Simulate latent network (chain):
    O_T <- simGraph_WS(nManifest)
    
    # Transform chain to go between factors:
    inds <- matrix(seq_len(sum(nIndicator)), max(nIndicator), nLatent)
    inds[duplicated(c(inds))] <- NA
    jig <- c(t(inds))
    jig <- order(jig[!is.na(jig)])
    O_T <- O_T[jig,jig]
    
    # Construct theta:
    Theta <- cov2cor(solve(eye(nManifest) - O_T))

    # Construct psi and sigma:    
    Psi <- matrix(0.25,nLatent,nLatent)
    diag(Psi) <- 1
    Sigma <- Lambda %*% Psi %*% t(Lambda) + Theta
    
    # Simulate data:
    Data <- rmvnorm(sampleSize, sigma=Sigma)
    
    # Fit model:
    t0 <- Sys.time()

    Res <- lvnetLasso(cov(Data), lassoMatrix = "omega_theta", sampleSize = nrow(Data),lambda = ifelse(Lambda==1,NA,0),
                      verbose = FALSE, refit = FALSE, criterion = "aic")
    
    timeDiff <- as.numeric(difftime(Sys.time(), t0,units = "mins"))

    Results <- lapply(c("AIC","BIC","EBIC"), function(x){

      if (x == "AIC"){
        Res <- lassoSelect(Res, aic, refit=FALSE)
      }
      
      if (x == "BIC"){
        Res <- lassoSelect(Res, bic, refit=FALSE)
      }
      
      if (x == "EBIC"){
        Res <- lassoSelect(Res, ebic, refit=FALSE)
      }
      
      ### STORE RESULTS ###
      SimulationResults <- data.frame(criterion = x)
      
      # time difference:
      SimulationResults$timeDiff <- timeDiff
      
      
      # Chisq:
      SimulationResults$chisq <- Res$best$fitMeasures$chisq
      
      # df:
      SimulationResults$df <- Res$best$fitMeasures$df
      
      # p-value:
      SimulationResults$pvalue <- Res$best$fitMeasures$pvalue
      
      # RMSEA:
      SimulationResults$rmsea <- Res$best$fitMeasures$rmsea
      
      # CFI:
      SimulationResults$cfi <- Res$best$fitMeasures$cfi
      
      # SRMR:
      SimulationResults$srmr <-  Res$best$fitMeasures$srmr
      
      # Omega_theta:
      res_omega_theta <- Res$best$matrices$omega_theta
      res_omega_theta[abs(res_omega_theta) < 1e-4] <- 0
      
      # Equal?
      SimulationResults$correctModel <- all((res_omega_theta == 0) == ( O_T == 0))
      
      # True positives:
      TruePos <- sum(res_omega_theta[upper.tri(res_omega_theta)] != 0 &  O_T[upper.tri( O_T)] != 0)
      
      # False pos:
      FalsePos <- sum(res_omega_theta[upper.tri(res_omega_theta)] != 0 &  O_T[upper.tri( O_T)] == 0)
      
      # True Neg:
      TrueNeg <- sum(res_omega_theta[upper.tri(res_omega_theta)] == 0 &  O_T[upper.tri( O_T)] == 0)
      
      # False Neg:
      FalseNeg <- sum(res_omega_theta[upper.tri(res_omega_theta)] == 0 &  O_T[upper.tri( O_T)] != 0)
      
      ### Sensitivity:
      SimulationResults$sensitivity <- TruePos / (TruePos + FalseNeg)
      
      # Specificity:
      SimulationResults$specificity <- TrueNeg / (TrueNeg + FalsePos)
      
      SimulationResults
      
    })
    
    Results <- do.call(rbind,Results)
    return(Results)
  }
)