#!/usr/bin/env Rscript
args <- commandArgs(trailingOnly=TRUE)
if (length(args)==0){
  args <- 1
}

# Needed packages:
library("methods")
library("lvnet")
library("mvtnorm")
library("dplyr")
library("ggplot2")
library("pracma")

# parSim function:
source("parSim.R")

# Extra functions
source("funs.R")

### RUN THE SIMULATION ###
parSim(
  # Conditions:
  # Single values can be changed into a vector to test multiple conditions
  sampleSize = c(50,100,250,500,1000), # sample size
  nLatent = 2, # Number of latents
  criterion = c("AIC","BIC","chisq","EBIC"), # Criterions to test
 
  # Setup:
  name = paste0("RNM_stepwise_run4_",args[1]),
  write = TRUE, # If FALSE, simulation results are stored to a data frame. Now, results are written to a text file.
  nCores = 16, # Number of cores to use. Set to 1 for debugging options
  reps = 10, # Number of times all conditions should be repeated. When using a high-performance cluster, my typical setup is to leave this to 1 and submit this file 100 times.
  export =  c("simGraph_WS","forcePositive"), # Functions that should be exported to the cluster
  
  # Codes:
  expression = {
    suppressMessages(library("lvnet"))
    suppressMessages( library("mvtnorm"))
    suppressMessages(library("dplyr"))
    suppressMessages(library("ggplot2"))
    suppressMessages(library("pracma"))
    suppressMessages(library("igraph"))
    
    # Set number of indicators per latent:
    nManifest <- 10
    if (nLatent==1){
      nIndicator <- 10
    } else if (nLatent == 2){
      nIndicator <- c(5,5)
    } else stop("Incorrect number of indicators")
    
    # Total number of possible edges:
    totEdges <- (nManifest*(nManifest-1)) / 2

    # Construct lambda:
    Lambda <- matrix(0,nManifest, nLatent)
    c <- 0
    for (j in 1:nLatent){
      Lambda[c + 1:nIndicator[j],j] <- 1
      c <- c + nIndicator[j]
    }
    
    # Simulate latent network (chain):
    O_T <- simGraph_WS(nManifest)
    
    # Transform chain to go between factors:
    if (nLatent==1){
      jig <- 1:10
    } else if (nLatent == 2){
      jig <- order(c(1,6,2,7,3,8,4,9,5,10))
    } else stop("Incorrect number of indicators")
    O_T <- O_T[jig,jig]
    
    # Construct theta:
    Theta <- cov2cor(solve(eye(nManifest) - O_T))

    # Construct psi and sigma:    
    Psi <- matrix(0.25,nLatent,nLatent)
    diag(Psi) <- 1
    Sigma <- Lambda %*% Psi %*% t(Lambda) + Theta
    
    # Simulate data:
    Data <- rmvnorm(sampleSize, sigma=Sigma)
    
    # Fit model:
    t0 <- Sys.time()
    Res <- lvnetSearch(cov(Data), matrix = "omega_theta", sampleSize = nrow(Data),lambda = ifelse(Lambda==1,NA,0),
                       start = "empty", criterion = tolower(criterion), verbose = FALSE, maxChange = 1)
    
    timeDiff <- as.numeric(difftime(Sys.time(), t0,units = "mins"))
    
    ### STORE RESULTS ###
    SimulationResults <- list()
    
    # time difference:
    SimulationResults$timeDiff <- timeDiff
    
    
    # Chisq:
    SimulationResults$chisq <- Res$best$fitMeasures$chisq
    
    # df:
    SimulationResults$df <- Res$best$fitMeasures$df
    
    # p-value:
    SimulationResults$pvalue <- Res$best$fitMeasures$pvalue
    
    # RMSEA:
    SimulationResults$rmsea <- Res$best$fitMeasures$rmsea
    
    # CFI:
    SimulationResults$cfi <- Res$best$fitMeasures$cfi
    
    # SRMR:
    SimulationResults$srmr <-  Res$best$fitMeasures$srmr
    
    # Omega_psi:
    res_omega_theta <- Res$best$matrices$omega_theta
    
    # Equal?
    SimulationResults$correctModel <- all((res_omega_theta == 0) == ( O_T == 0))
    
    # True positives:
    TruePos <- sum(res_omega_theta[upper.tri(res_omega_theta)] != 0 &  O_T[upper.tri( O_T)] != 0)
    
    # False pos:
    FalsePos <- sum(res_omega_theta[upper.tri(res_omega_theta)] != 0 &  O_T[upper.tri( O_T)] == 0)
    
    # True Neg:
    TrueNeg <- sum(res_omega_theta[upper.tri(res_omega_theta)] == 0 &  O_T[upper.tri( O_T)] == 0)
    
    # False Neg:
    FalseNeg <- sum(res_omega_theta[upper.tri(res_omega_theta)] == 0 &  O_T[upper.tri( O_T)] != 0)
    
    ### Sensitivity:
    SimulationResults$sensitivity <- TruePos / (TruePos + FalseNeg)
    
    # Specificity:
    SimulationResults$specificity <- TrueNeg / (TrueNeg + FalsePos)
    
    SimulationResults
  }
)